enum EventName {
  RefreshToken = 'refreshToken'
}

const tracker = anue.shared.get('extern.trackjs');
const trackError = (error: { message: string; detail: any }) => {
  if (tracker) {
    console.log('send error to TrackJS', error);
    tracker.error(error);
  }
};

/**
 * Dispatch a event to App. Web->App
 */
const send = (
  eventName: Anue.Utils.NativeEventName,
  payload: Record<string, any> = {}
) => {
  if (typeof native !== 'undefined') {
    try {
      switch (eventName) {
        case 'closeWebView':
          native.member_event(JSON.stringify({ action: 'anueCloseWebView' }));
          break;
        case 'anueRequestRefreshToken':
          native.member_event(
            JSON.stringify({ action: 'anueRequestRefreshToken' })
          );
          break;
        case 'anueMemberLevelChanged':
          native.member_event(
            JSON.stringify({ action: 'anueMemberLevelChanged' })
          );
          break;
        case 'anueAlert':
          native.member_event(
            JSON.stringify({
              action: 'anueAlert',
              title: payload.title || 'anueAlert is called',
              message: payload.message || 'anueAlert is called'
            })
          );
          break;
        case 'anueOpenByBrowser':
          if (!payload || !payload.href) {
            console.error('anueOpenByBrowser event must pass the href parameter');
          } else {
            native.member_event(
              JSON.stringify({
                action: 'anueOpenByBrowser',
                href: payload.href,
              })
            );
          }
          break;
        case 'anueGoToPaidNewsTab':
          native.member_event(
            JSON.stringify({ action: 'anueGoToPaidNewsTab' })
          );
          break;
        default:
        // TODO: Move the implementation to SDK from fe-portal.
        // native.event(JSON.stringify(payload));
      }
    } catch (e) {
      console.warn(`%cError occurs during communicate with APP`, e);
    }
  }
};

/**
 * Init the listener for App. App->Web
 */
const addListener = {
  webViewStatus() {
    if (typeof window !== 'undefined') {
      window.appCallWebCurrentStatus = (status: number) => {
        switch (status) {
          case 0:
            window.webViewStatus = 'onResume';
            break;
          case 1:
            window.webViewStatus = 'onPause';
            break;
          default:
            window.webViewStatus = status;
        }

        return 'appCallWebCurrentStatus is called from App';
      };
    }
  },
  webViewRefreshToken() {
    if (typeof window !== 'undefined') {
      console.log('%c[WebView] appCallWebRefreshToken', 'color: red;');
      window.appCallWebRefreshToken = (token: string) => {
        if (window._fetchingWebViewRefreshToken) {
          const event = new CustomEvent(EventName.RefreshToken, {
            detail: token
          });
          window.dispatchEvent(event);
        }

        return 'appCallWebRefreshToken is called from App';
      };
    }
  }
};

interface AnueNativeEvent extends Event {
  detail: string;
}

const get = {
  webViewStatus: () => window.webViewStatus,
  async webViewRefreshToken() {
    if (typeof window !== 'undefined') {
      window._fetchingWebViewRefreshToken = true;

      const webViewRefreshTokenTimeout = new Promise<void>(
        (resolve, reject) => {
          const t = setTimeout(() => {
            clearTimeout(t);
            window._fetchingWebViewRefreshToken = false;
            reject('timeout');
          }, 5000);
        }
      );

      const webViewRefreshTokenFetch = new Promise<Anue.Auth.RefreshTokenResponse | void>(
        (resolve, reject) => {
          function handleRefreshTokenResponse(event: AnueNativeEvent) {
            const token = event.detail;
            window._fetchingWebViewRefreshToken = false;

            if (token) {
              send('anueAlert', { title: 'anueAlert - Received token' });
              resolve({
                statusCode: 200,
                message: 'Renew the token from App',
                items: {
                  identityId: '',
                  token
                }
              });
            } else {
              trackError({
                message: 'Failed',
                detail: 'App Refresh token time out'
              });
              send('anueAlert', { title: 'anueAlert - Refresh token failed!' });
              reject();
            }

            window.removeEventListener(
              EventName.RefreshToken,
              handleRefreshTokenResponse as EventListener
            );
          }

          window.addEventListener(
            EventName.RefreshToken,
            handleRefreshTokenResponse as EventListener
          );
          send('anueRequestRefreshToken');
        }
      );

      return Promise.race<Anue.Auth.RefreshTokenResponse | void>([
        webViewRefreshTokenFetch,
        webViewRefreshTokenTimeout
      ])
        .then(fulfill => fulfill)
        .catch(reject => {
          if (reject === 'timeout') {
            trackError({
              message: 'Time out',
              detail: 'App Refresh token time out'
            });
          }
        });
    }
  }
};

const webview = {
  send,
  addListener,
  get
};

anue.shared.set('utils.webview', webview);

export default webview;
